#' BaiNgIC
#'
#' This function estimates Bai and Ng’s information criteria.
#' @param xdat A dataframe containing panel data.
#' @param factor_max The maximum number of the common factors.
#' @keywords Panel data
#' @export
#' @examples
#' BaiNgIC(assault_dat[,2:ncol(assault_dat)])$ic2
#'
#'
BaiNgIC <- function(xdat, factor_max = 8){
  n <- ncol(xdat)
  t <- nrow(xdat)

  xdat1 <- xdat - matrix(apply(xdat, 2, mean), ncol = n, nrow = t, byrow = TRUE)
  xdat1 <- as.matrix(xdat1)
  if(t>n){
    xdat1 <- t(xdat1)
  }
  XXdat <- xdat1 %*% t(xdat1)/(t * n)
  svd_out <- svd(t(XXdat))
  va <- svd_out$d

  m <- nrow(XXdat)
  v0 <- as.numeric()
  for(k in 1:(factor_max + 1)){
    v0[k] <- sum(va[k:m])
  }

  ks <- 0:factor_max
  c <- min(n, t)

  ic1 <- log(v0) + ks * ((n + t)/ (n * t)) * log((n * t) / (2 * (n + t)))
  ic2 <- log(v0) + ks * ((n + t)/ (n * t)) * log(c)
  ic3 <- log(v0) + ks * (log(c)/c)

  fact_ic1 <- which(ic1 == min(ic1)) -1
  fact_ic2 <- which(ic2 == min(ic2)) - 1
  fact_ic3 <- which(ic3 == min(ic3)) - 1

  my_list <- list(ic1 = fact_ic1, ic2 = fact_ic2, ic3 = fact_ic3)
  return(my_list)
}
