
panel_check <- function(xdat, lag = 1){
  n <- ncol(xdat)
  t <- nrow(xdat)

  miny <- apply(xdat, 2, min)
  store_mat <- matrix(NA, nrow = t, ncol = 4)
  for(ser in 1:n){
    yy <- xdat[,ser]
    testmod <- lm(yy~seq_along(yy))
    # tval <- coef(summary(testmod))[2,3]
    resids <- resid(testmod)
    xx <- model.matrix(testmod)
    sig <- NeweyWestvcov(resids, lag)
    sig <- sig %*% diag(solve(t(xx) %*% xx))
    tstats <- coef(summary(testmod))[,1]/sqrt(sig)
    store_mat[ser, 1] <- ser
    store_mat[ser, 2] <- miny[ser]
    store_mat[ser, 3] <- testmod$coefficients[2]
    store_mat[ser, 4] <- tstats[2]
  }
  colnames(store_mat) <- c("series", "minimum y", "beta", "t stat")
  return(store_mat)
}

ones <- function(n1, n2 = n1){
  out <- matrix(1, nrow = n1, ncol = n2)
  return(out)

}

rnorm_mat <- function(n1, n2 = n1){
  norm_vars <- rnorm(n1 * n2)
  out <- matrix(norm_vars, nrow = n1, ncol = n2, byrow = FALSE)
  return(out)
}


# xdat dataframe with t rows and n columns of data with no other id
# num_factor number of factors

# pc  xdat <- dps

logtregress <- function(xdat, p = floor(t/3)){
  n <- ncol(xdat)
  t <- nrow(xdat)
  mu <- apply(xdat, 1, mean)

  xdat1 <- xdat/kronecker(matrix(1, 1, n), mu)
  xdat2 <- (xdat1 -1) * (xdat1 -1)
  mu2 <- apply(xdat2, 1, mean)
  xdat3 <- log(1/mu2 * mu2[1])
  rht <- xdat3[(p + 1):t]
  trd <- p + seq_along(rht)
  ltrd <- log(trd)

  rht <- rht - (2 * log(ltrd))
  dat <- cbind.data.frame(rht, ltrd)
  logtmod <- lm(dat)
  res <- residuals(logtmod)
  res <- res - mean(res)
  lags <- floor(t^(1/3))
  sig <- NeweyWestvcov(res, lags)
  xx_logtmod <- model.matrix(logtmod)
  sig <- sig %*% diag(solve(t(xx_logtmod) %*% xx_logtmod))
  tstats <- coef(summary(logtmod))[,1]/sqrt(sig)

  my_list <- list("b" = coef(summary(logtmod))[2,1], "t stat" = tstats[2])
  return(my_list)

}

ones <- function(n1, n2 = n1){
  out <- matrix(1, nrow = n1, ncol = n2)
  return(out)

}

zeros <- function(n1, n2 = n1){
  out <- matrix(0, nrow = n1, ncol = n2)
  return(out)

}

rnorm_mat <- function(n1, n2 = n1){
  norm_vars <- rnorm(n1 * n2)
  out <- matrix(norm_vars, nrow = n1, ncol = n2, byrow = FALSE)
  return(out)
}


# pc no standardization version (not recommended to use)
pcnostand <-function(xdat, num_factor = 1){

  xdat <- as.matrix.data.frame(xdat)

  t <- dim(xdat)[1]
  n <- dim(xdat)[2]

  xdat1 <- as.matrix(xdat)
  if(t>n){
    xdat1 <- t(xdat1)
    turn = 1
  } else {
    turn = 0
  }

  n <- ncol(xdat1)
  t <- nrow(xdat1)
  vx <- (xdat1 %*% t(xdat1))/n
  ve <- svd(vx)$v

  if(num_factor > 0){
    ve2 <- ve[,1:num_factor]
    f <- ve2 * sqrt(t)
    lambda <- t(xdat1) %*% f/t
    com <- f %*% t(lambda)
    if(turn == 0){
      vv <- xdat - com
      # on purpose to use vv for weak sigma convergence test. vv includes a fixed effect.
    } else {
      com <- t(com)
      vv <- xdat - com
      temp <- f
      f <- lambda
      lambda <- temp
    }

    tt <- ncol(vv)
    vnt <-  matrix(1, nrow = tt, ncol = 1) - (apply(vv,2,var)  / apply(xdat,2,var));
  } else {
    e <- xdat1
    f <- matrix(0, nrow = t, 1)
    lambda <- matrix(rep(0,n), nrow = n, ncol = 1)
    v <- xdat
    vnt <- apply(xdat, 2, var)
  }



  my_list <- list( f1 = f, lambda1 = lambda, v1 = vnt, e1 = vv)
  return(my_list)
}












