#' olshac
#'
#' This function creates does ols with HAC errors.
#' @param df A dataframe containing only data used in regression.
#' @param lag Number of lags to use in computation.
#' @keywords Panel data
#' @export
#' @examples
#' beta_2_facts <- olshac(data, 6)
#'
#'
olshac <- function(df, lag = 1){
  tmp.mod <- lm(df)
  t <- nobs(tmp.mod)
  if(lag != 1){
    lag <- round(t^(1/3))
  }
  resids <- resid(tmp.mod)
  xx <- model.matrix(tmp.mod)
  res <- resids * xx
  sig <- NeweyWestvcov(res, lag)
  xtx1 <- solve(crossprod(xx))
  beta <- tmp.mod$coefficients
  sig <- sqrt(diag(xtx1 %*% sig %*% xtx1 * (t)))
  tstat <- coef(summary(tmp.mod))[,1]/sig
  round(cbind(beta, sig, tstat), 4)
}
