#' pc
#'
#' This function estimates the principal components of a panel.
#' @param xdat A dataframe containing panel data.
#' @param num_factor The number of the common factors.
#' @param stand Standardize the data first, default is true
#' @keywords Panel data
#' @export
#' @examples
#' pc(pdp, 8)
#'
#'
pc <-function(xdat, num_factor = 1, stand = TRUE){

  xdat <- as.matrix.data.frame(xdat)

  t <- dim(xdat)[1]
  n <- dim(xdat)[2]

  if(stand == TRUE){
    xdat2 <- xdat/ matrix(apply(xdat, 2, sd), ncol = n, nrow = t, byrow = TRUE)
    xdat1 <- xdat2 - matrix(apply(xdat2, 2, mean), ncol = n, nrow = t, byrow = TRUE)
    xdat1 <- as.matrix(xdat1)
  } else {
    xdat2 <- xdat
    xdat1 <- as.matrix(xdat)

  }


  if(t>n){
    xdat1 <- t(xdat1)
    turn = 1
  } else {
    turn = 0
  }

  n <- ncol(xdat1)
  t <- nrow(xdat1)
  vx <- (xdat1 %*% t(xdat1))/n
  ve <- svd(vx)$v

  if(num_factor > 0){
    ve2 <- ve[,1:num_factor]
    f <- ve2 * sqrt(t)
    lambda <- t(xdat1) %*% f/t
    com <- f %*% t(lambda)
    if(turn == 0){
      vv <- xdat2 - com
      # on purpose to use vv for weak sigma convergence test. vv includes a fixed effect.
    } else {
      com <- t(com)
      vv <- xdat2 - com
      temp <- f
      f <- lambda
      lambda <- temp
    }

    tt <- ncol(vv)
    vnt <-  matrix(1, nrow = tt, ncol = 1) - (apply(vv,2,var)  / apply(xdat2,2,var));
  } else {
    e <- xdat1
    f <- matrix(0, nrow = t, 1)
    lambda <- matrix(rep(0,n), nrow = n, ncol = 1)
    v <- xdat
    vnt <- apply(xdat, 2, var)
  }



  my_list <- list( f1 = f, lambda1 = lambda, v1 = vnt, e1 = vv)
  return(my_list)
}
