#' weak_converge
#'
#' This function creates does ols with HAC errors.
#' @param xdat A dataframe containing only data used in regression.
#' @param lag Number of lags to use in computation.
#' @keywords Panel data
#' @export
#' @examples
#' weak_converge(violent_dat, 3)
#'
#'
weak_converge <- function(xdat, lag = 1){
  n <- ncol(xdat)
  t <- nrow(xdat)

  mx <- apply(xdat, 1, mean)
  xdat1 <- xdat - mx
  xdat2 <- xdat1 * xdat1
  xdat3 <- apply(xdat2, 1, mean)

  trd <- 1:t
  x_df <- cbind.data.frame(xdat3, trd)
  mod_weak_c <- lm(x_df)
  resids <- resid(mod_weak_c)
  xx <- model.matrix(mod_weak_c)
  sig <- NeweyWestvcov(resids, lag)
  sig <- sig %*% diag(solve(t(xx) %*% xx))
  tstats <- coef(summary(mod_weak_c))[,1]/sqrt(sig)
  # return(tstats)
  my_list <- list("phi" = mod_weak_c$coefficients[2], "tstats" = tstats[2])
  return(my_list)

}
