## ----chapter 1
## ----ch101-------------------------------------------------------------------------------------------------------------------------------------------------------

library(readr)
mat <- read_csv("data/MATn46_t39.csv", skip = 2)

## ----ch103-------------------------------------------------------------------------------------------------------------------------------------------------------

library(stringr)

# rename columns
mat_nms <- colnames(mat) 
mat_nms <- str_replace(mat_nms, "\\?", "Year")
mat_nms <- str_replace(mat_nms, "@", "Empty")
mat_nms <- str_replace(mat_nms, "%", "Empty2")
colnames(mat) <- mat_nms

## ----chapter 2
## ----ch201, echo=FALSE-------------------------------------------------------------------------------------------------------------------------------------------

rm(list = ls())
library(kableExtra)
library(tidyverse)
library(PDEwCF)

# load data
data("MATn46_t39")

## ----ch203-------------------------------------------------------------------------------------------------------------------------------------------------------
head(MATn46_t39)

mat <- read_csv("data/MATn46_t39.csv", skip = 2)
# rename columns
mat_nms <- colnames(mat) 
mat_nms <- str_replace(mat_nms, "\\?", "Year")
mat_nms <- str_replace(mat_nms, "@", "Empty")
mat_nms <- str_replace(mat_nms, "%", "Empty2")
colnames(mat) <- mat_nms

## ----ch205a, echo=FALSE------------------------------------------------------------------------------------------------------------------------------------------

head(mat[,1:8], 5)

## ----ch206-------------------------------------------------------------------------------------------------------------------------------------------------------

# make the data tidy (narrow format)
MATn46_t39 <- mat %>%
  dplyr::select(-contains("Empty")) %>%
  gather(key = inflation, value = price, -Year)

## ----ch208, warning=FALSE, message=FALSE-------------------------------------------------------------------------------------------------------------------------

by_time <- MATn46_t39 %>%
  group_by(Year) %>%
  summarise(Mean = mean(price),
            Median = median(price),
            Min = min(price),
            Max = max(price),
            Std.dev = sd(price),
            IQR = IQR(price),
            N = n())

## ----ch210, fig.cap="Quick time series plot of some inflation series", fig.pos= 'H'------------------------------------------------------------------------------

# plot a subsample of the data
MATn46_t39 %>%
  filter(inflation == "Food and nonalcoholic beverages purchased for off-premises consumption" |
           inflation == "Alcoholic beverages purchased for off-premises consumption" |
           inflation == "Food produced and consumed on farms"  |
           inflation ==  "Clothing" ) %>%
  ggplot() +
  geom_line(aes(x = Year, y = price, color = inflation)) + theme_minimal()

## ----ch211, fig.cap="Quick density plot of some inflation series", fig.pos= 'H'----------------------------------------------------------------------------------

MATn46_t39 %>%
  filter(inflation == "Food and nonalcoholic beverages purchased for off-premises consumption" |
           inflation == "Alcoholic beverages purchased for off-premises consumption" |
           inflation == "Food produced and consumed on farms"  |
           inflation ==  "Clothing" ) %>%
  ggplot() +
  geom_density(aes(x = price, color = inflation)) +
  theme_minimal() +
  facet_wrap(~inflation, scales = "free")  + # creates a grid of plots
  theme(legend.position = "none") # removes legend
  

## ----chapter 3
## ----ch301-------------------------------------------------------------------------------------------------------------------------------------------------------

# load the data
data("crime_data")
panel_list <- c("violent", "murder", "robbery", 
                "rape", "assault", "property", 
                "burglary", "larceny", "motorv")  

## ----ch303-------------------------------------------------------------------------------------------------------------------------------------------------------

crime_data <- crime_data %>%
  group_by(Crime_Type, State) %>%
  mutate(dlncrime = lncrime - dplyr::lag(lncrime, 1),  # dy
         lncrime_plus = lncrime/sd(lncrime, na.rm = TRUE), # y+
         dlncrime_plus = dlncrime/sd(dlncrime, na.rm = TRUE)) %>%  # dy+
  ungroup() %>%
  gather(key = variable, value = vals, -Year, -Crime_Type, -State, -id) %>%
  dplyr::select(-id) %>%
  spread(key = State, value = vals) %>%
  arrange(Crime_Type)
  
var_list <- c("lncrime", "lncrime_plus", "dlncrime", "dlncrime_plus")

## ----dummyloop, eval=FALSE---------------------------------------------------------------------------------------------------------------------------------------
## 
## for(i in 1:5){
##   # some code here
##   print(i) # prints loop counter i
## }
## 


## ----ch305-------------------------------------------------------------------------------------------------------------------------------------------------------

store_crime_ic <- matrix(NA, nrow = length(panel_list), ncol = 4)

# loop through the crime panels
for(i in 1:length(panel_list)){
  # use this value in the next loop to filter the data
  filt_val = panel_list[i]
  # loop through the four variables
  for(v in 1:length(var_list)){
    # get the variable to use for input         
    v_val = var_list[v]                 
    
    # organize the data to fit the input for the function BaiNgIC
    dat <- crime_data %>%
      # jointly filter the panel and variable
      filter(Crime_Type == filt_val & variable == v_val) %>%
      # remove unnecessary columns
      dplyr::select(-Year, -Crime_Type, -variable) %>%       
      na.omit()   # remove NA values                                           
    
# find the number of factors based on the IC2 information criteria and store it
    store_crime_ic[i, v] <- BaiNgIC(dat)$ic2  # this is a scalar output
  } # close loop v

} # close loop i

tab31 <- data.frame(store_crime_ic)
colnames(tab31) <- var_list
tab31 <- cbind(panel_list, tab31)

## ----ch307-------------------------------------------------------------------------------------------------------------------------------------------------------

# load the data
data("MATn46_t39")

inflation_data <- MATn46_t39 %>%
  group_by(inflation) %>%
  mutate(dp = (price - dplyr::lag(price))/ dplyr::lag(price), # create inflation
       dp1 = (dp - lag(dp)  ), # whiten data
       pdp = dp/sd(dp, na.rm = TRUE), # standardize
       ddp = dp1/sd(dp1, na.rm = TRUE)) %>% # standardize whitened data
  ungroup()

## ----ch308-------------------------------------------------------------------------------------------------------------------------------------------------------

# undertake a window analysis

ic2_window_analysis <- function(xdat, start_year, roll_window = 25, white_coef = 0.5){
  store_dxs <- as.numeric() # for results
  store_xxs <- as.numeric() 
  year_labs <- as.character()
  labstore <- as.character()
  maxyear <- max(xdat$Year, na.rm = TRUE)
  WW <- maxyear - roll_window - start_year + 1
  xdat2 <- xdat %>%
    dplyr::select(Year, inflation, price) %>%
    group_by(inflation) %>%
    # then compute measures
    mutate(dp = (price - dplyr::lag(price))/ dplyr::lag(price)) %>% # create inflation
    ungroup()
  
  for(w in 1:(WW)){
    Year_1 = start_year + w   
    Year_25 = Year_1 + roll_window - 1
    year_labs <- paste0(as.character(Year_1), "-", as.character(Year_25))
    
    # get years of data
    xdat3 <- xdat2 %>%
      filter(between(Year, Year_1, Year_25))  %>%
      group_by(inflation) %>%
      mutate(dx = dp - (white_coef * lag(dp, 1)), # whiten data
             xxs = dx/sd(dx, na.rm = TRUE),       # standardize whitened data
             dxs = dp/sd(dp, na.rm = TRUE)) %>%   # standardize
      ungroup() 
    xdat4 <- xdat3 %>%
      dplyr::select(Year, inflation, dxs) %>%
      spread(key = inflation, value = dxs) %>%
      na.omit() %>%
      dplyr::select(-Year)
    
    # the ic
    store_dxs[w] <-  BaiNgIC(xdat4)$ic2
    
    xdat5 <- xdat3 %>%
      dplyr::select(Year, inflation, xxs) %>%
      spread(key = inflation, value = xxs) %>%
      na.omit() %>%
      dplyr::select(-Year)  
    store_xxs[w] <-  BaiNgIC(xdat5)$ic2 
    labstore[w] <- year_labs
    
  } # w 
  out <- cbind.data.frame(sample = labstore, 
                          standardized = store_dxs, 
                          white_standardized = store_xxs)
  return(out)
}  # close function

## ----ch309-------------------------------------------------------------------------------------------------------------------------------------------------------

ic2_robust <- ic2_window_analysis(inflation_data, 1978, 25)

ic2_robust %>%
  gather(key = Series, value = number_of_factors, -sample) %>% 
  # -sample leaves sample as its own column
  ggplot() +
  geom_point(aes(x = sample, y = number_of_factors, 
                 shape = Series, color = Series), size = 2.5, alpha = 0.6) +
  # some minor dressing
  theme_minimal() +
  labs(y = "Estimated number factors", x = "Sample") +
  theme(axis.text.x=element_text(angle=90,hjust=1)) 


## ----chapter 4
## ----ch401-------------------------------------------------------------------------------------------------------------------------------------------------------

# prepare data for analysis
inflation_data <- MATn46_t39 %>%
  group_by(inflation) %>%
  mutate(dp = (price - dplyr::lag(price))/ dplyr::lag(price), # create inflation
       dp1 = (dp - lag(dp)  ), # whiten data
       pdp = dp/sd(dp, na.rm = TRUE), # standardize
       ddp = dp1/sd(dp1, na.rm = TRUE)) %>% # standardize whitened data
  ungroup()

## ----ch403-------------------------------------------------------------------------------------------------------------------------------------------------------

# Spread the data wide
ddp <- inflation_data %>%
  dplyr::select(Year, inflation, ddp) %>%
  na.omit() %>%
  spread(key = inflation, value = ddp) %>%
  dplyr::select(-Year) 

# Use Bai Ng information criteria
k <- BaiNgIC(ddp)$ic2

## ----ch404-------------------------------------------------------------------------------------------------------------------------------------------------------

pdp <- inflation_data %>%
  dplyr::select(Year, inflation, pdp) %>%
  na.omit() %>%
  spread(key = inflation, value = pdp) %>%
  dplyr::select(-Year) 

# pc estimation
fact.est.pdp <- pc(pdp, k)

f.pdp <- fact.est.pdp$f1      # factor
l.pdp <- fact.est.pdp$lambda1 # loadings
e.pdp <- fact.est.pdp$v1      # R2 
v.pdp <- fact.est.pdp$e1      # fixed effect + idiosyncratic term

## ----ch405-------------------------------------------------------------------------------------------------------------------------------------------------------

data("MATint99")

## ----ch406, echo=FALSE, fig.cap="Interest rate data"-------------------------------------------------------------------------------------------------------------

MATint99 %>%
  ggplot() +
  geom_line(aes(x = Date, y = Interestrate, color = Country)) +
  theme_minimal() + labs(y = "Interest rates")

## ----ch407, warning=FALSE, fig.cap="Figure 4.5", fig.height=3----------------------------------------------------------------------------------------------------

# find country with greatest variance
var.big <- MATint99 %>%
  group_by(Country) %>%
  mutate(var= var(Interestrate),
         n = n()) %>%
  ungroup() %>%
  mutate(max = ifelse(max(var) == var, 1, 0)) %>% 
  filter(max == 1) %>%
  dplyr::select(Country) %>%
  unique() %>%
  as.character()

# raw data
sq <- MATint99 %>%
  dplyr::select(Date, Country, Interestrate) %>%
  na.omit() %>%
  spread(key = Country, value = Interestrate) %>%
  dplyr::select(-Date) 

fact.est.sq <- pc(sq, 1, stand = FALSE)

# repeat with standardization
sq1 <- MATint99 %>%
  group_by(Country) %>%
  mutate(sq1 = Interestrate / sd(Interestrate)) %>%
  ungroup() %>%
  dplyr::select(Date, Country, sq1) %>%
  na.omit() %>%
  spread(key = Country, value = sq1) %>%
  dplyr::select(-Date) 

# pc estimation standarized data
fact.est.sq1 <- pc(sq1, 1)

Label.int.rate <- paste0("Interest rate - ", var.big)
# prepare data for plotting
ff.dta <- MATint99 %>%
  filter(Country == var.big) %>%
  bind_cols(f1 = -fact.est.sq$f1, f2 = -fact.est.sq1$f1) %>%
  dplyr::select(-Country) %>%
  gather(key = col, value = ff, -Date) %>%
  group_by(col) %>%
  mutate(ff =  ff- mean(ff),
         ff = ff/sd(ff),
  # Labels for plots
         col = ifelse(col == "Interestrate", 
                      Label.int.rate, ifelse(col == "f2", 
                                             "PC Factor after standardization", 
                                             "PC Factor before standardization"))) %>%
  ungroup() 

# Plot both cases here 
ff.dta %>% 
  ggplot() +
  geom_line(aes(x = Date, y = ff, color = col)) +
  geom_point(aes(x = Date, y = ff, color = col, shape = col)) +
  theme_minimal() +
  labs(y = "") + theme(legend.title = element_blank())


## ----chapter 5
## ----ch501-------------------------------------------------------------------------------------------------------------------------------------------------------

data("MATn3_t39")

## ----ch502, echo=FALSE-------------------------------------------------------------------------------------------------------------------------------------------
head(MATn3_t39, 5)

## ----ch502b, echo=FALSE, fig.cap="Three sub inflation indices", fig.height=3-------------------------------------------------------------------------------------

MATn3_t39 %>%
  ggplot() +
  geom_line(aes(x = Year, y = price, color = inflation)) +
  theme_minimal() +
  labs(y = "Price index") + theme(legend.title = element_blank())

## ----ch503-------------------------------------------------------------------------------------------------------------------------------------------------------

index_3_list <- c("Durable", "Nondurable", "Service")

# prepare data for analysis
inflation_data <- MATn46_t39 %>%
  bind_rows(MATn3_t39) %>%
  group_by(inflation) %>%
  mutate(dp = (price - dplyr::lag(price))/ dplyr::lag(price), # create inflation
       d2p = (dp - lag(dp)  ), 
       dps = dp/sd(dp, na.rm = TRUE), 
       d2ps = d2p/sd(d2p, na.rm = TRUE)) %>% 
  rename(p1 = price) %>%
  ungroup()

# spread and find factors
dps <- inflation_data %>%
  dplyr::select(Year, inflation, dps) %>%
  filter(!inflation %in% index_3_list) %>% # remove the extra 3 categories
  na.omit() %>%
  spread(key = inflation, value = dps) %>%
  dplyr::select(-Year) 

k_dps <- BaiNgIC(dps, 8)$ic2

d2ps <- inflation_data %>%
  dplyr::select(Year, inflation, d2ps) %>%
  filter(!inflation %in% index_3_list) %>% # remove the extra 3 categories
  na.omit() %>%
  spread(key = inflation, value = d2ps) %>%
  dplyr::select(-Year) 

k_d2ps <- BaiNgIC(d2ps, 8)$ic2

# for later
n <- ncol(d2ps)
bigT <- nrow(d2ps)

## ----ch504-------------------------------------------------------------------------------------------------------------------------------------------------------

store_bn <- matrix(NA, nrow = n, ncol = 2)
nms <- colnames(d2ps)
for(i in 1:n){
  if(i == 1){
    xx <- d2ps[,2:n]
  }
  if(i >1 & i == n){
    xx <- d2ps[,-i]
  }
  if(i == n){
    xx <- d2ps[,1:(n-1)]
  } 
  x1 <- cbind.data.frame(1,d2ps[,i])
  x1 <- as.matrix(x1)
  prx <- defactor(as.matrix(xx), as.matrix(x1))
  
  store_bn[i,1] <- i
  store_bn[i,2] <- BaiNgIC(prx, 8)$ic2
}
colnames(store_bn) <- c("Column", "Factors")

out <- cbind.data.frame(Name = nms, store_bn)

## ----ch506-------------------------------------------------------------------------------------------------------------------------------------------------------

store_bn <- matrix(NA, nrow = n, ncol = 11)
nms <- colnames(d2ps)
for(i in 1:n){
  if(i == 1){
    xx <- d2ps[,2:n]
  }
  if(i >1 & i == n){
    xx <- d2ps[,-i]
  }
  if(i == n){
    xx <- d2ps[,1:(n-1)]
  } 
  store_bn[i,1] <- i
  for(j in 1:10){
      x2 <- xx[j:bigT,]
      x3 <- cbind.data.frame(1,d2ps[j:bigT,i])
      x3 <- as.matrix(x3)
      prx <- defactor(as.matrix(x2), as.matrix(x3))
      
      store_bn[i,j + 1] <- BaiNgIC(prx, 8)$ic2
  } # j
} # i

colnames(store_bn) <- c("Column", paste0("SS", 1:10))

out_robust <- cbind.data.frame(Name = nms, store_bn)

## ----ch508, warning=FALSE----------------------------------------------------------------------------------------------------------------------------------------

d2p <- inflation_data %>%
  dplyr::select(Year, inflation, d2p) %>%
  filter(!inflation %in% index_3_list) %>%
  na.omit()

md2p <- d2p %>%
  group_by(Year) %>%
  summarise(md2p = mean(d2p))

d3p <- inflation_data %>%
  dplyr::select(Year, inflation, dp3 = dp, d2p3 = d2p, d23ps = d2ps) %>%
  filter(inflation %in% index_3_list) %>%
  na.omit()

## ----ch509-------------------------------------------------------------------------------------------------------------------------------------------------------

d2p3 <- d3p %>% 
  na.omit() %>%
  dplyr::select(Year, inflation, d2p3) %>%
  spread(key = inflation, value = d2p3) %>%
  dplyr::select(-Year) # make as mat

store_m_bn <- matrix(NA, nrow = 10, ncol = 5)

for(j in 1:10){
  store_m_bn[j, 1] <- j
  md2p_mat <- as.matrix(md2p[j:bigT,])
  d2ps_mat <- as.matrix(d2ps[j:bigT,])  

# try various combinations
  
  x1 <- cbind.data.frame(1,d2p3[j:bigT,]) %>% as.matrix() # all three vars as in text   
  x12 <- cbind.data.frame(1,d2p3[j:bigT,-3]) %>% as.matrix()  
  x13 <- cbind.data.frame(1,d2p3[j:bigT,-2]) %>% as.matrix()
  x23 <- cbind.data.frame(1,d2p3[j:bigT,-1]) %>% as.matrix()
  
  hx = x1 %*% solve(crossprod(x1)) %*% t(x1) %*% md2p_mat
  hx12 = x12 %*% solve(crossprod(x12)) %*% t(x12) %*% md2p_mat
  hx13 = x13 %*% solve(crossprod(x13)) %*% t(x13) %*% md2p_mat
  hx23 = x23 %*% solve(crossprod(x23)) %*% t(x23) %*% md2p_mat
  
  x1 <- cbind.data.frame(1,hx) %>% as.matrix()
  x12 <- cbind.data.frame(1,hx12) %>% as.matrix()
  x13 <- cbind.data.frame(1,hx13) %>% as.matrix()
  x23 <- cbind.data.frame(1,hx23) %>% as.matrix()
    
  prx <- d2ps_mat - x1 %*% solve(crossprod(x1)) %*% t(x1) %*% d2ps_mat
  prx12 <- d2ps_mat - x12 %*% solve(crossprod(x12)) %*% t(x12) %*% d2ps_mat
  prx13 <- d2ps_mat - x13 %*% solve(crossprod(x13)) %*% t(x13) %*% d2ps_mat
  prx23 <- d2ps_mat - x23 %*% solve(crossprod(x23)) %*% t(x23) %*% d2ps_mat
  
  store_m_bn[j, 2] <- BaiNgIC(prx12, 8)$ic2
  store_m_bn[j, 3] <- BaiNgIC(prx13, 8)$ic2
  store_m_bn[j, 4] <- BaiNgIC(prx23, 8)$ic2
  store_m_bn[j, 5] <- BaiNgIC(prx, 8)$ic2
} # j

colnames(store_m_bn) <- c("Sub sample", paste(index_3_list[c(1,2)], collapse = ", "), 
                          paste(index_3_list[c(1,3)], collapse = ", "), 
                          paste(index_3_list[c(2,3)], collapse = ", "),  "All variables")

out_multiple <- cbind.data.frame(store_m_bn)

## ----ch510, echo=FALSE-------------------------------------------------------------------------------------------------------------------------------------------

kable(out_multiple,  booktabs = T, escape = FALSE, digits = 2, caption = "Examination of multiple variables as single factor for inflation data") %>%
  kableExtra::kable_styling("striped", full_width = F) %>%
  kableExtra::kable_styling(position = "center")

## ----chapter 6
## ----ch0601------------------------------------------------------------------------------------------------------------------------------------------------------

# load data
data("spot")
data("price")

## ----ch0602------------------------------------------------------------------------------------------------------------------------------------------------------

# price data
us_dta <- price %>%
  filter(Country == "US") %>%
  dplyr::select(Date, US = price)

p_dta <- price %>%
  filter(Country != "US") %>%
  left_join(us_dta, by = "Date") %>%
  mutate(p = log(price/US)) %>%
  group_by(Country) %>%
  mutate(dp = p - dplyr::lag(p, 1),
         d2p = dp -  dplyr::lag(dp, 1),
         sdp = dp/sd(dp, na.rm = TRUE),
         sd2p = d2p/sd(d2p, na.rm = TRUE)) 

## ----ch0603------------------------------------------------------------------------------------------------------------------------------------------------------

dp_dta <- p_dta %>%
  dplyr::select(Date, Country, dp) %>%
  ungroup() %>%
  spread(key = Country, value = dp) %>%
  na.omit()

d2p_dta <- p_dta %>%
  dplyr::select(Date, Country, d2p) %>%
  ungroup() %>%
  spread(key = Country, value = d2p) %>%
  na.omit()

sdp_dta <- p_dta %>%
  dplyr::select(Date, Country, sdp) %>%
  ungroup() %>%
  spread(key = Country, value = sdp) %>%
  na.omit()

sd2p_dta <- p_dta %>%
  dplyr::select(Date, Country, sd2p) %>%
  ungroup() %>%
  spread(key = Country, value = sd2p) %>%
  na.omit()

# make the spot rates data
s_dta <- spot %>%
  filter(Date <= max(dp_dta$Date)) %>%
  mutate(s = log(price)) %>%
  group_by(Country) %>%
  mutate(ds = s - dplyr::lag(s, 1)) %>%
  dplyr::select(Date, Country, s) %>%
  ungroup()

euro_dta <- s_dta %>%
  filter(Country == "EURO") %>%
  dplyr::select(Date, EURO = s)

deu_dta <- s_dta %>%
  filter(Country != "EURO") %>%  
  left_join(euro_dta, by = "Date") %>%
  mutate(seu = s - EURO) %>%
  group_by(Country) %>%
  mutate(deu = seu - dplyr::lag(seu, 1)) %>%  
  dplyr::select(Date, Country, deu) %>%
  spread(key = Country, value = deu) %>%
  na.omit()

ds_dta <- spot %>%
  filter(Date <= max(dp_dta$Date)) %>%
  mutate(s = log(price)) %>%
  group_by(Country) %>%
  mutate(ds = s - dplyr::lag(s, 1)) %>%
  dplyr::select(Date, Country, ds) %>%
  spread(key = Country, value = ds)  %>%
  dplyr::select(-US) %>%
  na.omit()

# clean up workspace
rm( "euro_dta", "s_dta", "spot", "us_dta", "price") 

## ----ch0604------------------------------------------------------------------------------------------------------------------------------------------------------

# Estimation the common-dynamic relationship

# csa based factors
mdus = apply(ds_dta[-1], 1, mean)
mdeu =apply(deu_dta[-1], 1, mean)
mdp =apply(dp_dta[-1], 1, mean)

dta.1 <- cbind.data.frame(mdus, mdp)
dta.2 <- cbind.data.frame(mdus, mdp, mdeu)
beta_2_facts <- olshac(dta.1, 6)
betas_3_facts <- olshac(dta.2, 6)

## ----ch0605------------------------------------------------------------------------------------------------------------------------------------------------------

T2 <- dim(dp_dta)[1]
n2 <- dim(dp_dta)[2]

# loop through dataframe in samples of 50, whiten data, find IC2
pp_dta <- p_dta %>%
  dplyr::select(-sdp, -sd2p) %>%
  filter(Date > min(p_dta$Date)) %>%
  group_by(Country) %>%
  mutate(rn = row_number()) %>% # for each country
  ungroup() %>%
  dplyr::select(rn, everything())

mx_rn <- max(pp_dta$rn)

ic_store <- matrix(NA, 50, 3)
colnames(ic_store) <- c("Sample", "sdp", "sd2p")

for(i in 1:50){
  tmp_sdp_dta <- pp_dta  %>%
    filter(rn >= i) %>% 
    group_by(Country) %>%
    mutate(sdp = dp/sd(dp, na.rm = TRUE)) %>%
    dplyr::select(Date, Country, sdp) %>%
    ungroup() %>%
    spread(key = Country, value = sdp) %>%
    dplyr::select(-Date )
  
  tmp_sd2p_dta <- pp_dta  %>%
    filter(rn >= i+1) %>% 
    group_by(Country) %>%
    mutate(sd2p = d2p/sd(d2p, na.rm = TRUE)) %>%
    dplyr::select(Date, Country, sd2p) %>%
    ungroup() %>%
    spread(key = Country, value = sd2p) %>%
    dplyr::select(-Date )
  
  ic_store[i,1] <- i
  ic_store[i,2] <- BaiNgIC(tmp_sdp_dta)$ic2
  ic_store[i,3] <- BaiNgIC(tmp_sd2p_dta)$ic2

}

tab_factors_sdp <- summary(as.factor(ic_store[,2]))
tab_factors_sdp2 <- summary(as.factor(ic_store[,3]))

## ----ch0607------------------------------------------------------------------------------------------------------------------------------------------------------

xx1 <- cbind.data.frame(1, mdp, mdus)
xx1 <- as.matrix(xx1)
xtx1 <- solve(crossprod(xx1))
dsc <- as.matrix(ds_dta[,2:ncol(ds_dta)]) - xx1 %*% xtx1 %*% t(xx1) %*% as.matrix(ds_dta[,2:ncol(ds_dta)] ) 
dpc <- as.matrix(dp_dta[,2:ncol(dp_dta)]) - xx1 %*% xtx1 %*% t(xx1) %*% as.matrix(dp_dta[,2:ncol(dp_dta)] )
bcce1 <- sum(sum(dsc*dpc))/sum(sum(dpc*dpc))

xx2 <- cbind.data.frame(1, mdp, mdus, mdeu)
xx2 <- as.matrix(xx2)
xtx12 <- solve(crossprod(xx2))

dsc <- as.matrix(ds_dta[,2:ncol(ds_dta)]) - xx2 %*% xtx12 %*% t(xx2) %*% as.matrix(ds_dta[,2:ncol(ds_dta)] ) 
dpc <- as.matrix(dp_dta[,2:ncol(dp_dta)]) - xx2 %*% xtx12 %*% t(xx2) %*% as.matrix(dp_dta[,2:ncol(dp_dta)] )
bcce2 <- sum(sum(dsc*dpc))/sum(sum(dpc*dpc))

## ----ch0608------------------------------------------------------------------------------------------------------------------------------------------------------

sds1 <- ds_dta %>%
  gather(key = Country, value = ds, -Date) %>%
  group_by(Country) %>%
  mutate(m = mean(ds),
         sd= sd(ds),
         sds = (ds - m)/sd) %>%
  dplyr::select(Country, Date, sds) %>%
  spread(key = Country, value = sds) %>%
  dplyr::select(-Date) %>% as.matrix()

sdp1 <- dp_dta %>%
  gather(key = Country, value = dp, -Date) %>%
  group_by(Country) %>%
  mutate(sdp = (dp - mean(dp))/sd(dp)) %>%
  dplyr::select(Country, Date, sdp) %>%
  spread(key = Country, value = sdp) %>%
  dplyr::select(-Date) %>% as.matrix()

# get factors
fs <- pc(sds1, 2)$f1
fp <- pc(sdp1, 1)$f1

# create
ymat <- ds_dta[ ,-1 ]
xmat <- cbind(1, fs, fp)
y_3 <- defactor(y_mat = ymat, x_mat = xmat)
ymat <- dp_dta[ ,-1 ]
x_3 <- defactor(y_mat = ymat, x_mat = xmat)

be1 <- sum(sum(x_3*y_3))/sum(sum(x_3*x_3))

xmat <- cbind(1, fs, fp, mdp, mdus, mdeu)
ymat <- ds_dta[ ,-1 ]
y_3b <- defactor(y_mat = ymat, x_mat = xmat)

ymat <- dp_dta[ ,-1 ]
x_3b <- defactor(y_mat = ymat, x_mat = xmat)

be3 <- sum(sum(x_3b*y_3b))/sum(sum(x_3b*x_3b))

# Panel Robust t-ratio =====
re <- y_3b - x_3b * be3
re = re * x_3b
xq <- apply(re, 2, sum)
xq <-  crossprod(xq)
invxx <- 1/sum(sum(x_3b * x_3b));
xq <- invxx %*% xq %*% invxx
xq <- sqrt(diag(xq))
tr <- be3 / xq
ghs_3_facts <- cbind(b = be3, tstat = tr)

## ----ch0610------------------------------------------------------------------------------------------------------------------------------------------------------

bb <- be3
ds_mat <- as.matrix(ds_dta[ ,-1 ])
dp_mat <- as.matrix(dp_dta[ ,-1 ])

tol <- 10^-10
iters <- 500
for(i in 1:iters){
  rs <- ds_mat - dp_mat * bb
  rss <- stand_mat(rs)
  k3 = BaiNgIC(rss, 8)$ic2
  fr <- pc(rs, k3)$f1
  ff <- cbind(1, fr)
  # ff <- cbind(1, fr, mdp, mdus, mdeu)
  y4 <- defactor(ds_mat, ff)
  x4 <- defactor(dp_mat, ff)
  
  be4 <- sum(sum(x4*y4))/sum(sum(x4*x4))
  if((be4 - bb) < tol){
    break
  }
  bb <- be4
  }

## ----ch0612------------------------------------------------------------------------------------------------------------------------------------------------------

for(i in 1:iters){
  rs <- ds_mat - dp_mat * bb
  rss <- stand_mat(rs)
  k3 = BaiNgIC(rss, 8)$ic2
  fr <- pc(rs, k3)$f1
  # ff <- cbind(1, fr)
  ff <- cbind(1, fr, mdp, mdus, mdeu)
  y4 <- defactor(ds_mat, ff)
  x4 <- defactor(dp_mat, ff)
  
  be4 <- sum(sum(x4*y4))/sum(sum(x4*x4))
  if((be4 - bb) < tol){
    break
  }
  bb <- be4
  }

## ----ch06plm-----------------------------------------------------------------------------------------------------------------------------------------------------

library(plm)

dta_dp_n <- dp_dta %>%
  gather(key = Country, value = dp, -Date) 

deu_dta_n <- deu_dta %>%
  gather(key = Country, value = deu, -Date) 

ds_dta_n  <- ds_dta %>%
  gather(key = Country, value = ds, -Date) %>% 
  left_join(dta_dp_n, by = c("Date", "Country")) %>%
  left_join(deu_dta_n, by = c("Date", "Country")) %>%
  pdata.frame(index = c("Country", "Date") )
  
mod_pcce <- pcce(ds ~ dp, data = ds_dta_n, model="p")

# for mean group use model = "mg"
# mod_mgcce <- pcce(ds ~ dp, data = ds_dta_n, model="mg")

summary(mod_pcce)

## ----chapter 7
## ----ch0701------------------------------------------------------------------------------------------------------------------------------------------------------

violent_mat <- read_csv("data/violent_mat.csv")

violent_mat_nms <- colnames(violent_mat) 
violent_mat_nms <- str_replace( violent_mat_nms, "%", "Year")
colnames(violent_mat) <- violent_mat_nms

# organize data
violent_dat <- violent_mat %>%
  gather(key = State, value = Crimes, - Year) %>%
  mutate(lncrimes = log(Crimes)) %>%
  dplyr::select(-Crimes) %>%
  spread(key = State, value = lncrimes)  %>%
  dplyr::select(-Year) %>%
  data.frame()

## ----ch0702------------------------------------------------------------------------------------------------------------------------------------------------------

wc_3_out <- weak_converge(violent_dat, 3)
wc_4_out <- weak_converge(violent_dat, 4)

store_mat <- matrix(NA, nrow = 9, ncol = 6)
store_mat[1,1] <- round(100 * wc_3_out$phi, 3)
store_mat[1,2] <- round(wc_3_out$tstats, 3)
store_mat[1,3] <- round(wc_4_out$tstats, 3)

# final file list
final_list <- c("violent_dat")
panel_list <- c("violent")

## ----ch0704------------------------------------------------------------------------------------------------------------------------------------------------------

out_pc_violent <- pc(violent_dat, 1)
f1 <- as.matrix(out_pc_violent$f1) # pc factors
xx <- matrix(c(matrix(1, nrow = nrow(violent_dat)), f1), ncol = 2)
b <- solve(t(xx) %*% xx) %*% t(xx)  %*% as.matrix(violent_dat)
b2 <- t(as.matrix(b[2,]))
z <- violent_dat - f1 %*% b2

wc_violent_3 <- weak_converge(z, 3)
wc_violent_4 <- weak_converge(z, 4)
store_mat[1,4] <- round(100 * wc_violent_3$phi, 3)
store_mat[1,5] <- round(wc_violent_3$tstats, 3)
store_mat[1,6] <- round(wc_violent_4$tstats, 3)

## ----ch0703, ,echo=FALSE, message=FALSE--------------------------------------------------------------------------------------------------------------------------

r <- 2
name_list <- c("murder", "robbery", "rape", "assault", "property", "burglary", "larceny", "motorv")
for (i in name_list) {
  filename <- paste0("data/",i,"_mat.csv")
  crime_mat <- read_csv(filename, 
                        skip = 1)
# rename columns
  mat_nms <- colnames(crime_mat) 
  mat_nms <- str_replace(mat_nms, "%", "Year")
  mat_nms <- str_replace(mat_nms, "@", "Empty")
  colnames(crime_mat) <- mat_nms

# organize data for weak sigma convergence regression test
  crime_dat <- crime_mat %>%
    gather(key = State, value = Crimes, - Year, -Empty) %>%
    mutate(lncrimes = log(Crimes)) %>%
    dplyr::select(-Crimes, -Empty) %>%
    spread(key = State, value = lncrimes)  %>%
    dplyr::select(-Year) %>%
    data.frame()

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
  wc_3_out <- weak_converge(crime_dat, 3)
  wc_4_out <- weak_converge(crime_dat, 4)

  store_mat[r,1] <- round(100 * wc_3_out$phi, 3)
  store_mat[r,2] <- round(wc_3_out$tstats, 3)
  store_mat[r,3] <- round(wc_4_out$tstats, 3)
  
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# pc version  
  
out_pc_crime <- pc(crime_dat, 1)
f1 <- as.matrix(out_pc_crime$f1) # pc factors
xx <- matrix(c(matrix(1, nrow = nrow(crime_dat)), f1), ncol = 2)
b <- solve(t(xx) %*% xx) %*% t(xx)  %*% as.matrix(crime_dat)
b2 <- t(as.matrix(b[2,]))
z <- crime_dat - f1 %*% b2

wc_crime_3 <- weak_converge(z, 3)
wc_crime_4 <- weak_converge(z, 4)
store_mat[r,4] <- round(100 * wc_crime_3$phi, 3)
store_mat[r,5] <- round(wc_crime_3$tstats, 3)
store_mat[r,6] <- round(wc_crime_4$tstats, 3)
  
  
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  
  r <- r + 1
  keep_file_names <- paste0(i, "_dat")
  assign(keep_file_names, crime_dat) 
  final_list <- c(final_list, keep_file_names)
  panel_list <- c(panel_list, i)
}

# basic output
csa_out <- cbind.data.frame(panel_list, store_mat)
colnames(csa_out) <- c("Series", "CSA phi x 100", "t (lags = 3)", "t (lags = 4)", "PC phi x 100", "t (lags = 3)", "t (lags = 4)")

# years_nms <- crime_mat$Year

## ----ch0706------------------------------------------------------------------------------------------------------------------------------------------------------

store_dta_props <- matrix(NA, nrow = length(final_list), ncol = 4)
r <- 1

for(i in final_list){
  data_rel <- get(i)  # assigns i to data_rel
  
  # how many negative
  my <- apply(data_rel, 2, min)
  percent_neg <- mean(ifelse(my < 0, 1, 0))
  store_dta_props[r, 2] <- percent_neg

  # number trending  
  trd <- 1:nrow(data_rel)
  ols_store <- matrix(NA, nrow = ncol(data_rel), ncol = 3)
  for(ii in 1:ncol(data_rel)){
    mod1dta <- cbind.data.frame(y = data_rel[,ii], trd) # make dataframe
    mod1 <- lm(mod1dta)
    res <- mod1$residuals
    sig <- NeweyWestvcov(res, 3)
    xx <- model.matrix(mod1)
    xtx1 <- solve(crossprod(xx))
    sig <- c(sig) * diag(xtx1)
    tstat <- mod1$coefficients/sqrt(sig)  
    ols_store[ii, 1] <- ii 
    ols_store[ii, 2] <- mod1$coefficients[2]
    ols_store[ii, 3] <- tstat[2]
  }
  lessthan <- mean(ifelse(ols_store[,3] <= -1.65, 1, 0))
  store_dta_props[r, 1] <- lessthan
  modlogt <- logtregress(data_rel) 
  store_dta_props[r, 3] <- modlogt$b
  store_dta_props[r, 4] <- modlogt$`t stat`

  r <- r + 1
}

## ----ch0708------------------------------------------------------------------------------------------------------------------------------------------------------

library(ConvergenceClubs)

data("filteredGDP")
clubs <- findClubs(filteredGDP, dataCols=2:35, unit_names = 1, refCol=35,time_trim = 1/3, cstar = 0, HACmethod = "FQSB")
summary(clubs)

